@@ -5,31 +5,42 @@ use warnings;
 use 5.0100;
 
 # ABSTRACT: OpenSMILES format reader and writer
-our $VERSION = '0.4.6'; # VERSION
+our $VERSION = '0.5.0'; # VERSION
 
 require Exporter;
 our @ISA = qw( Exporter );
 our @EXPORT_OK = qw(
     clean_chiral_centers
     is_aromatic
+    is_chiral
+    mirror
 );
 
+use List::Util qw(any);
+
+sub is_chiral($);
+sub is_chiral_tetrahedral($);
+sub mirror($);
+
 # Removes chiral setting from tetrahedral chiral centers with less than
 # four distinct neighbours. Returns the affected atoms.
 #
 # CAVEAT: disregards anomers
 # TODO: check other chiral centers
-# TODO: take hcount into consideration, see GH#2
 sub clean_chiral_centers($$)
 {
     my( $moiety, $color_sub ) = @_;
 
     my @affected;
     for my $atom ($moiety->vertices) {
-        next if !$atom->{chirality};
-        next if $moiety->degree($atom) != 4;
+        next unless is_chiral_tetrahedral( $atom );
+
+        my $hcount = exists $atom->{hcount} ? $atom->{hcount} : 0;
+        next if $moiety->degree($atom) + $hcount != 4;
+
         my %colors = map { ($color_sub->( $_ ) => 1) }
-                         $moiety->neighbours($atom);
+                         $moiety->neighbours($atom),
+                         ( { symbol => 'H' } ) x $hcount;
         next if scalar keys %colors == 4;
         delete $atom->{chirality};
         push @affected, $atom;
@@ -43,14 +54,49 @@ sub is_aromatic($)
     return $atom->{symbol} ne ucfirst $atom->{symbol};
 }
 
-# CAVEAT: requires output from non-raw parsing due to GH#2
+sub is_chiral($)
+{
+    my( $what ) = @_;
+    if( ref $what eq 'HASH' ) { # Single atom
+        return exists $what->{chirality};
+    } else {                    # Graph representing moiety
+        return any { is_chiral( $_ ) } $what->vertices;
+    }
+}
+
+sub is_chiral_tetrahedral($)
+{
+    my( $what ) = @_;
+    if( ref $what eq 'HASH' ) { # Single atom
+        return $what->{chirality} && $what->{chirality} =~ /^@@?$/
+    } else {                    # Graph representing moiety
+        return any { is_chiral_tetrahedral( $_ ) } $what->vertices;
+    }
+}
+
+sub mirror($)
+{
+    my( $what ) = @_;
+    if( ref $what eq 'HASH' ) { # Single atom
+        # FIXME: currently dealing only with tetrahedral chiral centers
+        if( is_chiral_tetrahedral( $what ) ) {
+            $what->{chirality} = $what->{chirality} eq '@' ? '@@' : '@';
+        }
+    } else {
+        for ($what->vertices) {
+            mirror( $_ );
+        }
+    }
+}
+
+# CAVEAT: requires output from non-raw parsing due issue similar to GH#2
 sub _validate($@)
 {
     my( $moiety, $color_sub ) = @_;
 
     for my $atom (sort { $a->{number} <=> $b->{number} } $moiety->vertices) {
         # TODO: AL chiral centers also have to be checked
-        if( $atom->{chirality} && $atom->{chirality} =~ /^@@?$/ ) {
+        if( is_chiral_tetrahedral( $atom ) ) {
             if( $moiety->degree($atom) < 4 ) {
                 # FIXME: tetrahedral allenes are false-positives
                 warn sprintf 'chiral center %s(%d) has %d bonds while ' .
@@ -74,7 +120,7 @@ sub _validate($@)
         }
 
         # Warn about unmarked tetrahedral chiral centers
-        if( !$atom->{chirality} && $moiety->degree($atom) == 4 ) {
+        if( !is_chiral( $atom ) && $moiety->degree( $atom ) == 4 ) {
             my $color_sub_local = $color_sub;
             if( !$color_sub_local ) {
                 $color_sub_local = sub { return $_[0]->{symbol} };
